@@ -1,11 +1,11 @@
-# Copyright: Damien Elmes <anki@ichi2.net>
+# Copyright: Ankitects Pty Ltd and contributors
 # License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html
 
 from anki import version as _version
 
 import getpass
 import sys
-import optparse
+import argparse
 import tempfile
 import builtins
 import locale
@@ -137,8 +137,19 @@ def setupLang(pm, app, force=None):
     # gettext
     _gtrans = gettext.translation(
         'anki', dir, languages=[lang], fallback=True)
-    builtins.__dict__['_'] = _gtrans.gettext
-    builtins.__dict__['ngettext'] = _gtrans.ngettext
+    def fn__(arg):
+        print("accessing _ without importing from anki.lang will break in the future")
+        print("".join(traceback.format_stack()[-2]))
+        from anki.lang import _
+        return _(arg)
+    def fn_ngettext(a, b, c):
+        print("accessing ngettext without importing from anki.lang will break in the future")
+        print("".join(traceback.format_stack()[-2]))
+        from anki.lang import ngettext
+        return ngettext(a, b, c)
+
+    builtins.__dict__['_'] = fn__
+    builtins.__dict__['ngettext'] = fn_ngettext
     anki.lang.setLang(lang, local=False)
     if lang in ("he","ar","fa"):
         app.setLayoutDirection(Qt.RightToLeft)
@@ -225,12 +236,12 @@ def parseArgs(argv):
     # as there's no such profile
     if isMac and len(argv) > 1 and argv[1].startswith("-psn"):
         argv = [argv[0]]
-    parser = optparse.OptionParser(version="%prog " + appVersion)
-    parser.usage = "%prog [OPTIONS] [file to import]"
-    parser.add_option("-b", "--base", help="path to base folder")
-    parser.add_option("-p", "--profile", help="profile name to load")
-    parser.add_option("-l", "--lang", help="interface language (en, de, etc)")
-    return parser.parse_args(argv[1:])
+    parser = argparse.ArgumentParser(description="Anki " + appVersion)
+    parser.usage = "%(prog)s [OPTIONS] [file to import]"
+    parser.add_argument("-b", "--base", help="path to base folder", default="")
+    parser.add_argument("-p", "--profile", help="profile name to load", default="")
+    parser.add_argument("-l", "--lang", help="interface language (en, de, etc)")
+    return parser.parse_known_args(argv[1:])
 
 def setupGL(pm):
     if isMac:
@@ -286,8 +297,6 @@ def _run(argv=None, exec=True):
 
     # parse args
     opts, args = parseArgs(argv)
-    opts.base = opts.base or ""
-    opts.profile = opts.profile or ""
 
     # profile manager
     from aqt.profiles import ProfileManager
@@ -347,9 +356,6 @@ environment points to a valid, writable folder.""")
             QMessageBox.critical(None, "Error", "Your video driver is incompatible. Please start Anki again, and Anki will switch to a slower, more compatible mode.")
             sys.exit(1)
 
-    # remaining pm init
-    pm.ensureProfile()
-
     # load the main window
     import aqt.main
     mw = aqt.main.AnkiQt(app, pm, opts, args)
