@@ -1,4 +1,4 @@
-# Copyright: Damien Elmes <anki@ichi2.net>
+# Copyright: Ankitects Pty Ltd and contributors
 # License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html
 
 # Profile handling
@@ -6,7 +6,6 @@
 # - Saves in pickles rather than json to easily store Qt window state.
 # - Saves in sqlite rather than a flat file so the config can't be corrupted
 
-import os
 import random
 import pickle
 import shutil
@@ -16,13 +15,14 @@ import re
 
 from aqt.qt import *
 from anki.db import DB
-from anki.utils import isMac, isWin, intTime, checksum
+from anki.utils import isMac, isWin, intTime
 import anki.lang
 from aqt.utils import showWarning
 from aqt import appHelpSite
 import aqt.forms
 from send2trash import send2trash
 import anki.sound
+from anki.lang import _
 
 metaConf = dict(
     ver=0,
@@ -126,16 +126,22 @@ a flash drive.""" % self.base)
     ######################################################################
 
     def profiles(self):
-        return sorted(x for x in
-            self.db.list("select name from profiles")
-            if x != "_global")
+        def names():
+            return self.db.list("select name from profiles where name != '_global'")
+
+        n = names()
+        if not n:
+            self._ensureProfile()
+            n = names()
+
+        return n
 
     def _unpickle(self, data):
         class Unpickler(pickle.Unpickler):
             def find_class(self, module, name):
                 if module == "PyQt5.sip":
                     try:
-                        import PyQt5.sip
+                        import PyQt5.sip # pylint: disable=unused-import
                     except:
                         # use old sip location
                         module = "sip"
@@ -226,9 +232,9 @@ details have been forgotten."""))
         # rename folder
         try:
             os.rename(oldFolder, newFolder)
-        except WindowsError as e:
+        except Exception as e:
             self.db.rollback()
-            if "Access is denied" in str(e):
+            if "WinError 5" in str(e):
                 showWarning(_("""\
 Anki could not rename your profile because it could not rename the profile \
 folder on disk. Please ensure you have permission to write to Documents/Anki \
@@ -338,12 +344,11 @@ create table if not exists profiles
         self._setDefaultLang()
         return True
 
-    def ensureProfile(self):
+    def _ensureProfile(self):
         "Create a new profile if none exists."
-        if self.firstRun:
-            self.create(_("User 1"))
-            p = os.path.join(self.base, "README.txt")
-            open(p, "w", encoding="utf8").write(_("""\
+        self.create(_("User 1"))
+        p = os.path.join(self.base, "README.txt")
+        open(p, "w", encoding="utf8").write(_("""\
 This folder stores all of your Anki data in a single location,
 to make backups easy. To tell Anki to use a different location,
 please see:
@@ -356,10 +361,6 @@ please see:
     # On first run, allow the user to choose the default language
 
     def _setDefaultLang(self):
-        # the dialog expects _ to be defined, but we're running before
-        # setupLang() has been called. so we create a dummy op for now
-        import builtins
-        builtins.__dict__['_'] = lambda x: x
         # create dialog
         class NoCloseDiag(QDialog):
             def reject(self):
@@ -425,10 +426,7 @@ please see:
 
         path = self._glPath()
         if not os.path.exists(path):
-            if qtminor >= 12:
-                return "auto"
-            else:
-                return "software"
+            return "software"
 
         mode = open(path, "r").read().strip()
 
