@@ -1,32 +1,45 @@
-# Copyright: Damien Elmes <anki@ichi2.net>
+# Copyright: Ankitects Pty Ltd and contributors
 # License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html
 
 from aqt.qt import *
 
 import time
 from anki.sound import Recorder
-from aqt.utils import saveGeom, restoreGeom
+from aqt.utils import saveGeom, restoreGeom, showWarning
+from anki.lang import _
+
+if not Recorder:
+    print("pyaudio not installed")
 
 def getAudio(parent, encode=True):
     "Record and return filename"
     # record first
+    if not Recorder:
+        showWarning("pyaudio not installed")
+        return
+
     r = Recorder()
     mb = QMessageBox(parent)
     restoreGeom(mb, "audioRecorder")
     mb.setWindowTitle("Anki")
     mb.setIconPixmap(QPixmap(":/icons/media-record.png"))
-    but = QPushButton(_("  Stop"))
-    but.setIcon(QIcon(":/icons/media-playback-stop.png"))
-    #but.setIconSize(QSize(32, 32))
+    but = QPushButton(_("Save"))
+    mb.addButton(but, QMessageBox.AcceptRole)
+    but = QPushButton(_("Cancel"))
     mb.addButton(but, QMessageBox.RejectRole)
+    mb.setEscapeButton(but)
     t = time.time()
     r.start()
+    time.sleep(r.startupDelay)
     QApplication.instance().processEvents()
     while not mb.clickedButton():
         txt =_("Recording...<br>Time: %0.1f")
         mb.setText(txt % (time.time() - t))
         mb.show()
         QApplication.instance().processEvents()
+    if mb.clickedButton() == mb.escapeButton():
+        r.stop()
+        return
     saveGeom(mb, "audioRecorder")
     # ensure at least a second captured
     while time.time() - t < 1:
